@@ -3,8 +3,9 @@ use crate::functional::{HelmholtzEnergyFunctional, DFT};
 use crate::geometry::Grid;
 use crate::solver::DFTSolver;
 use crate::weight_functions::WeightFunctionInfo;
-use feos_core::{Contributions, EosError, EosResult, EosUnit, EquationOfState, State};
-use log::{info, warn};
+use feos_core::{
+    log_result, Contributions, EosError, EosResult, EosUnit, EquationOfState, State, Verbosity,
+};
 use ndarray::{
     s, Array, Array1, ArrayBase, ArrayViewMut, ArrayViewMut1, Axis as Axis_nd, Data, Dimension,
     Ix1, Ix2, Ix3, RemoveAxis,
@@ -80,18 +81,9 @@ impl DFTSpecifications {
             .density
             .to_reduced(U::reference_density())?
             .sum_axis(Axis_nd(0));
-        let temperature = profile
-            .bulk
-            .temperature
-            .to_reduced(U::reference_temperature())?;
-        let mu_comp = profile
-            .chemical_potential
-            .to_reduced(U::reference_molar_energy())?
-            / temperature;
-        let mu_segment = profile.dft.component_index.mapv(|c| mu_comp[c]);
         Ok(Rc::new(Self::TotalMoles {
             total_moles: profile.integrate_reduced(rho),
-            chemical_potential: mu_segment,
+            chemical_potential: profile.reduced_chemical_potential()?,
         }))
     }
 }
@@ -106,7 +98,7 @@ impl<U: EosUnit, D: Dimension, F: HelmholtzEnergyFunctional> DFTSpecification<U,
         z: &Array1<f64>,
         _: &State<U, DFT<F>>,
     ) -> EosResult<Array1<f64>> {
-        let m = &profile.dft.m;
+        let m: &Array1<f64> = &profile.dft.m();
         Ok(match self {
             Self::ChemicalPotential => chemical_potential.clone(),
             Self::Moles { moles } => (moles / z).mapv(f64::ln) * m,
@@ -128,7 +120,6 @@ pub struct DFTProfile<U, D: Dimension, F> {
     pub dft: Rc<DFT<F>>,
     pub temperature: QuantityScalar<U>,
     pub density: QuantityArray<U, D::Larger>,
-    pub chemical_potential: QuantityArray1<U>,
     pub specification: Rc<dyn DFTSpecification<U, D, F>>,
     pub external_potential: Array<f64, D::Larger>,
     pub bulk: State<U, DFT<F>>,
@@ -198,39 +189,44 @@ where
         convolver: Rc<dyn Convolver<f64, D>>,
         bulk: &State<U, DFT<F>>,
         external_potential: Option<Array<f64, D::Larger>>,
+        density: Option<&QuantityArray<U, D::Larger>>,
     ) -> EosResult<Self> {
         let dft = bulk.eos.clone();
 
         // initialize external potential
         let external_potential = external_potential.unwrap_or_else(|| {
-            let mut n_grid = vec![dft.component_index.len()];
+            let mut n_grid = vec![dft.component_index().len()];
             grid.axes()
                 .iter()
                 .for_each(|&ax| n_grid.push(ax.grid.len()));
             Array::zeros(n_grid).into_dimensionality().unwrap()
         });
 
-        // intitialize density
-        let t = bulk.temperature.to_reduced(U::reference_temperature())?;
-        let bonds = dft
-            .bond_integrals(t, &external_potential, &convolver)
-            .mapv(f64::abs)
-            * (-&external_potential).mapv(f64::exp);
-        let mut density = Array::zeros(external_potential.raw_dim());
-        let bulk_density = bulk.partial_density.to_reduced(U::reference_density())?;
-        for (s, &c) in dft.component_index.iter().enumerate() {
-            density
-                .index_axis_mut(Axis_nd(0), s)
-                .assign(&(bonds.index_axis(Axis_nd(0), s).map(|is| is.min(1.0)) * bulk_density[c]));
-        }
+        // initialize density
+        let density = if let Some(density) = density {
+            density.clone()
+        } else {
+            let t = bulk.temperature.to_reduced(U::reference_temperature())?;
+            let bonds = dft
+                .bond_integrals(t, &external_potential, &convolver)
+                .mapv(f64::abs)
+                * (-&external_potential).mapv(f64::exp);
+            let mut density = Array::zeros(external_potential.raw_dim());
+            let bulk_density = bulk.partial_density.to_reduced(U::reference_density())?;
+            for (s, &c) in dft.component_index().iter().enumerate() {
+                density.index_axis_mut(Axis_nd(0), s).assign(
+                    &(bonds.index_axis(Axis_nd(0), s).map(|is| is.min(1.0)) * bulk_density[c]),
+                );
+            }
+            density * U::reference_density()
+        };
 
         Ok(Self {
             grid,
             convolver,
             dft: bulk.eos.clone(),
             temperature: bulk.temperature,
-            density: density * U::reference_density(),
-            chemical_potential: bulk.chemical_potential(Contributions::Total),
+            density,
             specification: Rc::new(DFTSpecifications::ChemicalPotential),
             external_potential,
             bulk: bulk.clone(),
@@ -291,7 +287,7 @@ where
         let mut d = rho.raw_dim();
         d[0] = self.dft.components();
         let mut density_comps = Array::zeros(d);
-        for (i, &j) in self.dft.component_index.iter().enumerate() {
+        for (i, &j) in self.dft.component_index().iter().enumerate() {
             density_comps
                 .index_axis_mut(Axis_nd(0), j)
                 .assign(&rho.index_axis(Axis_nd(0), i));
@@ -303,6 +299,28 @@ where
     pub fn total_moles(&self) -> QuantityScalar<U> {
         self.moles().sum()
     }
+
+    /// Return the chemical potential of the system
+    pub fn chemical_potential(&self) -> QuantityArray1<U> {
+        self.bulk.chemical_potential(Contributions::Total)
+    }
+
+    fn reduced_chemical_potential(&self) -> EosResult<Array1<f64>> {
+        let temperature = self
+            .bulk
+            .temperature
+            .to_reduced(U::reference_temperature())?;
+        let lambda_de_broglie = self
+            .dft
+            .ideal_gas()
+            .de_broglie_wavelength(temperature, self.bulk.eos.components());
+        let mu_comp = self
+            .chemical_potential()
+            .to_reduced(U::reference_molar_energy())?
+            / temperature
+            - lambda_de_broglie;
+        Ok(self.dft.component_index().mapv(|c| mu_comp[c]))
+    }
 }
 
 impl<U: Clone, D: Dimension, F> Clone for DFTProfile<U, D, F> {
@@ -313,7 +331,6 @@ impl<U: Clone, D: Dimension, F> Clone for DFTProfile<U, D, F> {
             dft: self.dft.clone(),
             temperature: self.temperature.clone(),
             density: self.density.clone(),
-            chemical_potential: self.chemical_potential.clone(),
             specification: self.specification.clone(),
             external_potential: self.external_potential.clone(),
             bulk: self.bulk.clone(),
@@ -348,17 +365,7 @@ where
         // Read from profile
         let temperature = self.temperature.to_reduced(U::reference_temperature())?;
         let density = self.density.to_reduced(U::reference_density())?;
-        let lambda_de_broglie = self
-            .dft
-            .functional
-            .ideal_gas()
-            .de_broglie_wavelength(temperature, self.bulk.eos.components());
-        let mu_comp = self
-            .chemical_potential
-            .to_reduced(U::reference_molar_energy())?
-            / temperature
-            - lambda_de_broglie;
-        let chemical_potential = self.dft.component_index.mapv(|i| mu_comp[i]);
+        let chemical_potential = self.reduced_chemical_potential()?;
         let mut bulk = self.bulk.clone();
 
         // Allocate residual vectors
@@ -391,11 +398,10 @@ where
         // Update bulk state
         let lambda_de_broglie = self
             .dft
-            .functional
             .ideal_gas()
             .de_broglie_wavelength(temperature, bulk.eos.components());
         let mut mu_comp = Array::zeros(bulk.eos.components());
-        for (s, &c) in self.dft.component_index.iter().enumerate() {
+        for (s, &c) in self.dft.component_index().iter().enumerate() {
             mu_comp[c] = chemical_potential[s];
         }
         bulk.update_chemical_potential(
@@ -416,7 +422,7 @@ where
             .bond_integrals(temperature, &dfdrho, &self.convolver);
 
         // Euler-Lagrange equation
-        let m = &self.dft.m;
+        let m = &self.dft.m();
         res_rho
             .outer_iter_mut()
             .zip(dfdrho.outer_iter())
@@ -473,17 +479,7 @@ where
         // Read from profile
         let temperature = self.temperature.to_reduced(U::reference_temperature())?;
         let mut density = self.density.to_reduced(U::reference_density())?;
-        let lambda_de_broglie = self
-            .dft
-            .functional
-            .ideal_gas()
-            .de_broglie_wavelength(temperature, self.bulk.eos.components());
-        let mut mu_comp = self
-            .chemical_potential
-            .to_reduced(U::reference_molar_energy())?
-            / temperature
-            - lambda_de_broglie;
-        let mut chemical_potential = self.dft.component_index.mapv(|i| mu_comp[i]);
+        let mut chemical_potential = self.reduced_chemical_potential()?;
         let mut bulk = self.bulk.clone();
 
         // initialize x-vector
@@ -520,19 +516,19 @@ where
         // Call solver(s)
         let (converged, iterations) = solver.solve(&mut x, &mut residual)?;
         if converged {
-            info!("DFT solved in {} iterations", iterations);
+            log_result!(solver.verbosity, "DFT solved in {} iterations", iterations);
         } else if debug {
-            warn!("DFT not converged in {} iterations", iterations);
+            log_result!(
+                solver.verbosity,
+                "DFT not converged in {} iterations",
+                iterations
+            );
         } else {
             return Err(EosError::NotConverged(String::from("DFT")));
         }
 
         // Update profile
         self.density = density * U::reference_density();
-        for (s, &c) in self.dft.component_index.iter().enumerate() {
-            mu_comp[c] = chemical_potential[s];
-        }
-        self.chemical_potential = mu_comp * temperature * U::reference_molar_energy();
         self.bulk = bulk;
 
         Ok(())
@@ -549,7 +545,7 @@ where
     pub fn entropy_density(&self, contributions: Contributions) -> EosResult<QuantityArray<U, D>> {
         // initialize convolver
         let t = self.temperature.to_reduced(U::reference_temperature())?;
-        let functional_contributions = self.dft.functional.contributions();
+        let functional_contributions = self.dft.contributions();
         let weight_functions: Vec<WeightFunctionInfo<Dual64>> = functional_contributions
             .iter()
             .map(|c| c.weight_functions(Dual64::from(t).derive()))
@@ -568,10 +564,19 @@ where
         Ok(self.integrate(&self.entropy_density(contributions)?))
     }
 
+    pub fn grand_potential_density(&self) -> EosResult<QuantityArray<U, D>> {
+        self.dft
+            .grand_potential_density(self.temperature, &self.density, &self.convolver)
+    }
+
+    pub fn grand_potential(&self) -> EosResult<QuantityScalar<U>> {
+        Ok(self.integrate(&self.grand_potential_density()?))
+    }
+
     pub fn internal_energy(&self, contributions: Contributions) -> EosResult<QuantityScalar<U>> {
         // initialize convolver
         let t = self.temperature.to_reduced(U::reference_temperature())?;
-        let functional_contributions = self.dft.functional.contributions();
+        let functional_contributions = self.dft.contributions();
         let weight_functions: Vec<WeightFunctionInfo<Dual64>> = functional_contributions
             .iter()
             .map(|c| c.weight_functions(Dual64::from(t).derive()))
